﻿using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Newtonsoft.Json;
using Sugar.Enties;
using Utils;

namespace PMS.NetCore.Controllers
{
    /// <summary>
    /// 系统配置
    /// </summary>
    [Produces("application/json;charset=UTF-8")]
    [Route("api/SystemConfiguration")]
    [ServiceFilter(typeof(Filter.TokenAuthorize))]
    public class SystemConfigurationController : BaseController
    {
        #region 结算方式
        /// <summary>
        /// 获取结算方式列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [Route("GetSettleTypeList")]
        public JsonResult GetSettleTypeList()
        {
            var manager = new settletypeManager();
            var list = manager.GetList();
            logger.Info($"【GetSettleTypeList】list:{JsonConvert.SerializeObject(list)}");
            return Json(list);
        }

        /// <summary>
        /// 根据结算方式编码获取结算方式实例
        /// </summary>
        /// <param name="dto">结算方式编码</param>
        /// <returns></returns>
        [HttpPost]
        [Route("GetSettleType")]
        public JsonResult GetSettleType([FromBody] SingleParamDTO dto)
        {
            logger.Info($"【GetSettleType】dto:{dto.value}");
            if (dto.value.IsNullOrEmpty())
            {
                return Json("结算方式编码不能为空");

            }
            var model = new settletypeManager().GetById(dto.value);
            logger.Info($"【GetSettleType】model:{JsonConvert.SerializeObject(model)}");
            return Json(model);

        }

        /// <summary>
        /// 结算方式添加
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("SettleTypeAdd")]
        public JsonResult SettleTypeAdd([FromBody] settletype dto)
        {
            logger.Info($"【SettleTypeAdd】dto:{JsonConvert.SerializeObject(dto)}");
            if(dto.csscode.IsNullOrEmpty())
            {
                return Json("结算方式编码不能为空");
            }
            var manager = new settletypeManager();
            var model = manager.GetById(dto.csscode);
            if(model!=null)
            {
                return Json("结算方式编码必须唯一");
            }
            var success = manager.Insert(dto);
            if(success)
            {
                return Json("插入成功");
            }
            else
            {
                return Json("插入失败");
            }
        }

        /// <summary>
        /// 结算方式修改
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("SettleTypeUpdate")]
        public JsonResult SettleTypeUpdate([FromBody] settletype dto)
        {
            logger.Info($"【SettleTypeUpdate】dto:{JsonConvert.SerializeObject(dto)}");
            if (dto.csscode.IsNullOrEmpty())
            {
                return Json("结算方式编码不能为空");
            }
            var manager = new settletypeManager();
            var model = manager.GetById(dto.csscode);
            if (model == null)
            {
                return Json("要修改的结算方式不存在");
            }
            var success = manager.Update(dto);
            if (success)
            {
                return Json("修改成功");
            }
            else
            {
                return Json("修改失败");
            }
        }

        /// <summary>
        /// 结算方式删除
        /// </summary>
        /// <param name="dto">结算方式主键值</param>
        /// <returns></returns>
        [HttpPost]
        [Route("SettleTypeDelete")]
        public JsonResult SettleTypeDelete([FromBody] SingleParamDTO dto)
        {
            logger.Info($"【SettleTypeDelete】dto:{JsonConvert.SerializeObject(dto)}");

            var manager = new settletypeManager();
            var model = manager.GetById(dto.value);
            if (model == null)
            {
                return Json("要删除的结算方式不存在");
            }
            var success = manager.Delete(m => m.csscode == dto.value);
            if (success)
            {
                return Json("删除成功");
            }
            else
            {
                return Json("删除失败");
            }
        }
        #endregion

        #region 会计科目
       /// <summary>
       /// 分页获取会计科目
       /// </summary>
       /// <param name="pageIndex">页码</param>
       /// <param name="pageSize">页面条数</param>
       /// <returns></returns>
        [HttpGet]
        [Route("GetCodeList")]
        public JsonResult GetCodeList(int pageIndex,int pageSize)
        {
            logger.Info($"【GetCodeList】pageIndex:{pageSize},pageSize:{pageSize}");
            if(pageSize<=0||pageIndex<=0)
            {
                return Json("页码或页面条数不能小于等于0");
            }
            var manager = new codeManager();
            var list = manager.Db.Queryable<code>().ToPageList(pageIndex,pageSize);
            logger.Info($"【GetCodeList】list:{JsonConvert.SerializeObject(list)}");
            return Json(list);
        }
        #endregion

        #region 业态档案
       /// <summary>
       /// 获取业态档案列表
       /// </summary>
       /// <returns></returns>
        [HttpPost]
        [Route("GetFdc_pm_operstateList")]
        public JsonResult GetFdc_pm_operstateList()
        {
            var manager = new fdc_pm_operstateManager();
            var list = manager.GetList();
            logger.Info($"【GetFdc_pm_operstateList】list:{JsonConvert.SerializeObject(list)}");
            return Json(list);
        }

        /// <summary>
        /// 获取业态档案实例
        /// </summary>
        /// <param name="dto">业态档案主键值</param>
        /// <returns></returns>
        [HttpPost]
        [Route("GetFdc_pm_operstate")]
        public JsonResult GetFdc_pm_operstate([FromBody] SingleParamDTO dto)
        {
            logger.Info($"【GetFdc_pm_operstate】dto:{JsonConvert.SerializeObject(dto)}");
            if (dto==null||dto.value.IsNullOrEmpty())
            {
                return Json("参数错误");
            }
            var manager = new fdc_pm_operstateManager();
            var model = manager.GetById(dto.value);
            logger.Info($"【GetFdc_pm_operstate】model:{JsonConvert.SerializeObject(model)}");
            return Json(model);
        }

        /// <summary>
        /// 业态档案添加
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("Fdc_pm_operstateAdd")]
        public JsonResult Fdc_pm_operstateAdd([FromBody] fdc_pm_operstate dto)
        {
            logger.Info($"【Fdc_pm_operstateAdd】dto:{JsonConvert.SerializeObject(dto)}");
            if(!dto.scode.IsInteger())
            {
                return Json("业态编码格式不正确，只能是数字形式");
            }
            if (dto.sname.IsNullOrEmpty())
            {
                return Json("业态名称不能为空");
            }
            if (!dto.fathercode.IsInteger())
            {
                return Json("上级节点编码格式不正确，只能是数字形式");
            }

            if(dto.scode==dto.fathercode)
            {
                return Json("不能设置自身为上级节点");
            }
            var manager = new fdc_pm_operstateManager();
            var model = manager.GetById(dto.scode);
            if(model!=null)
            {
                return Json("业态编码必须唯一");
            }
            var fatherModel = manager.GetById(dto.fathercode);
            if(fatherModel==null&&dto.fathercode!="0")
            {
                return Json("父节点不存在");
            }
            var success = manager.Insert(dto);
            logger.Info($"【Fdc_pm_operstateAdd】success:{success}");
            if (success)
            {
                return Json("添加成功");
            }
            else
            {
                return Json("添加失败");
            }
        }

        /// <summary>
        /// 业态档案修改
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("Fdc_pm_operstateUpdate")]
        public JsonResult Fdc_pm_operstateUpdate([FromBody] fdc_pm_operstate dto)
        {
            logger.Info($"【Fdc_pm_operstateUpdate】dto:{JsonConvert.SerializeObject(dto)}");
            if (!dto.scode.IsInteger())
            {
                return Json("业态编码格式不正确，只能是数字形式");
            }
            if (dto.sname.IsNullOrEmpty())
            {
                return Json("业态名称不能为空");
            }
            if (!dto.fathercode.IsInteger())
            {
                return Json("上级节点编码格式不正确，只能是数字形式");
            }
            if (dto.scode == dto.fathercode)
            {
                return Json("不能设置自身为上级节点");
            }
            var manager = new fdc_pm_operstateManager();
            var model = manager.GetById(dto.scode);
            if (model == null)
            {
                return Json("业态档案不存在");
            }
            var fatherModel = manager.GetById(dto.fathercode);
            if (fatherModel == null && dto.fathercode != "0")
            {
                return Json("父节点不存在");
            }
            model = dto;
            var success = manager.Update(model);
            logger.Info($"【Fdc_pm_operstateUpdate】success:{success}");
            if (success)
            {
                return Json("修改成功");
            }
            else
            {
                return Json("修改失败");
            }
        }

        /// <summary>
        /// 业态档案删除
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("Fdc_pm_operstateDelete")]
        public JsonResult Fdc_pm_operstateDelete([FromBody] SingleParamDTO dto)
        {
            if (dto.value.IsNullOrEmpty())
            {
                return Json("要删除的业态编码不能为空");
            }
            var manager = new fdc_pm_operstateManager();
            var model= manager.GetById(dto.value);
            if(model==null)
            {
                return Json("要删除的记录不存在");
            }
            var success = manager.CurrentDb.DeleteById(dto.value);
            if (success)
            {
                return Json("删除成功");
            }
            else
            {
                return Json("删除失败");
            }

        }
        #endregion

        #region 户型档案
        /// <summary>
        /// 获取户型档案列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [Route("GetFdc_pm_modelList")]
        public JsonResult GetFdc_pm_modelList()
        {
            var manager = new fdc_pm_modelManager();
            var list = manager.GetList();
            logger.Info($"【GetFdc_pm_modelList】list:{JsonConvert.SerializeObject(list)}");
            return Json(list);
        }

        /// <summary>
        /// 获取户型档案实例
        /// </summary>
        /// <param name="dto">户型档案主键值</param>
        /// <returns></returns>
        [HttpPost]
        [Route("GetFdc_pm_model")]
        public JsonResult GetFdc_pm_model([FromBody] SingleParamDTO dto)
        {
            logger.Info($"【GetFdc_pm_model】dto:{JsonConvert.SerializeObject(dto)}");
            if (dto == null || dto.value.IsNullOrEmpty())
            {
                return Json("参数错误");
            }
            var manager = new fdc_pm_modelManager();
            var model = manager.GetById(dto.value);
            logger.Info($"【GetFdc_pm_model】model:{JsonConvert.SerializeObject(model)}");
            return Json(model);
        }

        /// <summary>
        /// 户型档案添加
        /// </summary>
        /// <param name="dto">户型档案实例</param>
        /// <returns></returns>
        [HttpPost]
        [Route("Fdc_pm_modelAdd")]
        public JsonResult Fdc_pm_modelAdd([FromBody] fdc_pm_model dto)
        {
            logger.Info($"【Fdc_pm_modelAdd】dto:{JsonConvert.SerializeObject(dto)}");

            var manager = new fdc_pm_modelManager();
            var model = manager.GetById(dto.autoid);
            if (model != null)
            {
                return Json("编号必须唯一");
            }
            var model2 = new fdc_pm_operstateManager().GetById(dto.scode);
            if(model2==null)
            {
                return Json("所属业态不存在");
            }
            var success = manager.Insert(dto);
            logger.Info($"【Fdc_pm_modelAdd】success:{success}");
            if (success)
            {
                return Json("添加成功");
            }
            else
            {
                return Json("添加失败");
            }
        }


        /// <summary>
        /// 户型档案删除
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("Fdc_pm_modelDelete")]
        public JsonResult Fdc_pm_modelDelete([FromBody] SingleParamDTO dto)
        {
            if (dto.value.IsNullOrEmpty())
            {
                return Json("要删除的户型档案编号不能为空");
            }
            var manager = new fdc_pm_modelManager();
            var model = manager.GetById(dto.value);
            if (model == null)
            {
                return Json("要删除的记录不存在");
            }
            var success = manager.CurrentDb.DeleteById(dto.value);
            if (success)
            {
                return Json("删除成功");
            }
            else
            {
                return Json("删除失败");
            }

        }
        #endregion

        #region 朝向档案
        /// <summary>
        /// 获取朝向档案列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [Route("GetFdc_pm_directionList")]
        public JsonResult GetFdc_pm_directionList()
        {
            var manager = new fdc_pm_directionManager();
            var list = manager.GetList();
            logger.Info($"【GetFdc_pm_directionList】list:{JsonConvert.SerializeObject(list)}");
            return Json(list);
        }

        /// <summary>
        /// 获取户朝向案实例
        /// </summary>
        /// <param name="dto">朝向档案主键值</param>
        /// <returns></returns>
        [HttpPost]
        [Route("GetFdc_pm_direction")]
        public JsonResult GetFdc_pm_direction([FromBody] SingleParamDTO dto)
        {
            logger.Info($"【GetFdc_pm_direction】dto:{JsonConvert.SerializeObject(dto)}");
            if (dto == null || dto.value.IsNullOrEmpty())
            {
                return Json("参数错误");
            }
            var manager = new fdc_pm_directionManager();
            var model = manager.GetById(dto.value);
            logger.Info($"【GetFdc_pm_direction】model:{JsonConvert.SerializeObject(model)}");
            return Json(model);
        }

        /// <summary>
        /// 朝向档案添加
        /// </summary>
        /// <param name="dto">朝向档案实例</param>
        /// <returns></returns>
        [HttpPost]
        [Route("Fdc_pm_directionAdd")]
        public JsonResult Fdc_pm_directionAdd([FromBody] fdc_pm_direction dto)
        {
            logger.Info($"【Fdc_pm_directionAdd】dto:{JsonConvert.SerializeObject(dto)}");

            var manager = new fdc_pm_directionManager();

            if (dto.direction.IsNullOrEmpty())
            {
                return Json("朝向不能为空");
            }
            var success = manager.Insert(dto);
            logger.Info($"【Fdc_pm_directionAdd】success:{success}");
            if (success)
            {
                return Json("添加成功");
            }
            else
            {
                return Json("添加失败");
            }
        }


        /// <summary>
        /// 朝向档案删除
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("Fdc_pm_directionDelete")]
        public JsonResult Fdc_pm_directionDelete([FromBody] SingleParamDTO dto)
        {
            if (dto.value.IsNullOrEmpty())
            {
                return Json("要删除的户型档案编号不能为空");
            }
            var manager = new fdc_pm_directionManager();
            var model = manager.GetById(dto.value);
            if (model == null)
            {
                return Json("要删除的记录不存在");
            }
            var success = manager.CurrentDb.DeleteById(dto.value);
            if (success)
            {
                return Json("删除成功");
            }
            else
            {
                return Json("删除失败");
            }

        }
        #endregion

        #region 项目档案
        /// <summary>
        /// 获取项目档案列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [Route("GetFdc_pm_projectList")]
        public JsonResult GetFdc_pm_projectList()
        {
            var manager = new fdc_pm_projectManager();
            var list = manager.GetList();
            logger.Info($"【GetFdc_pm_projectList】list:{JsonConvert.SerializeObject(list)}");
            return Json(list);
        }

        /// <summary>
        /// 获取项目档案实例
        /// </summary>
        /// <param name="dto">项目档案主键值</param>
        /// <returns></returns>
        [HttpPost]
        [Route("GetFdc_pm_project")]
        public JsonResult GetFdc_pm_project([FromBody] SingleParamDTO dto)
        {
            logger.Info($"【GetFdc_pm_project】dto:{JsonConvert.SerializeObject(dto)}");
            if (dto == null || dto.value.IsNullOrEmpty())
            {
                return Json("参数错误");
            }
            var manager = new fdc_pm_projectManager();
            var model = manager.GetById(dto.value);
            logger.Info($"【GetFdc_pm_project】model:{JsonConvert.SerializeObject(model)}");
            return Json(model);
        }

        /// <summary>
        /// 项目档案添加
        /// </summary>
        /// <param name="dto">项目档案实例</param>
        /// <returns></returns>
        [HttpPost]
        [Route("Fdc_pm_projectAdd")]
        public JsonResult Fdc_pm_projectAdd([FromBody] fdc_pm_project dto)
        {
            logger.Info($"【Fdc_pm_projectAdd】dto:{JsonConvert.SerializeObject(dto)}");

            var manager = new fdc_pm_projectManager();

            if (dto.pcode.IsNullOrEmpty())
            {
                return Json("编码不能为空");
            }
            if (dto.pname.IsNullOrEmpty())
            {
                return Json("项目名称不能为空");
            }
            if (dto.fathercode.IsNullOrEmpty())    //父级项目为空
            {
                dto.fathercode = "0";      //赋默认值

            }
            else
            {
                var model = manager.GetById(dto.fathercode);
                if(model==null)
                {
                    return Json("父级项目不存在");
                }
            }
            var success = manager.Insert(dto);
            logger.Info($"【Fdc_pm_projectAdd】success:{success}");
            if (success)
            {
                return Json("添加成功");
            }
            else
            {
                return Json("添加失败");
            }
        }

        /// <summary>
        /// 项目档案修改
        /// </summary>
        /// <param name="dto">项目档案实例</param>
        /// <returns></returns>
        [HttpPost]
        [Route("Fdc_pm_projectUpdate")]
        public JsonResult Fdc_pm_projectUpdate([FromBody] fdc_pm_project dto)
        {
            logger.Info($"【Fdc_pm_projectUpdate】dto:{JsonConvert.SerializeObject(dto)}");

            var manager = new fdc_pm_projectManager();

            if (dto.pcode.IsNullOrEmpty())
            {
                return Json("编码不能为空");
            }
            var entity = manager.GetById(dto.pcode);
            if(entity==null)
            {
                return Json("不存在的项目档案");
            }
            if (dto.pname.IsNullOrEmpty())
            {
                return Json("项目名称不能为空");
            }
            if (dto.fathercode.IsNullOrEmpty())    //父级项目为空
            {
                dto.fathercode = "0";      //赋默认值

            }
            else
            {
                var model = manager.GetById(dto.fathercode);
                if (model == null)
                {
                    return Json("父级项目不存在");
                }
            }
            var success = manager.Insert(dto);
            logger.Info($"【Fdc_pm_projectUpdate】success:{success}");
            if (success)
            {
                return Json("添加成功");
            }
            else
            {
                return Json("添加失败");
            }
        }

        /// <summary>
        /// 项目档案删除
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("Fdc_pm_projectDelete")]
        public JsonResult Fdc_pm_projectDelete([FromBody] SingleParamDTO dto)
        {
            if (dto.value.IsNullOrEmpty())
            {
                return Json("要删除的项目档案编号不能为空");
            }
            var manager = new fdc_pm_projectManager();
            var model = manager.GetById(dto.value);
            if (model == null)
            {
                return Json("要删除的记录不存在");
            }
            var success = manager.CurrentDb.DeleteById(dto.value);
            if (success)
            {
                return Json("删除成功");
            }
            else
            {
                return Json("删除失败");
            }

        }
        #endregion
    }
}
